@@ -2,10 +2,13 @@
 Renders the command line on the console.
 (Redraws parts of the input line that were changed.)
 """
+
+from __future__ import annotations
+
 from asyncio import FIRST_COMPLETED, Future, ensure_future, sleep, wait
 from collections import deque
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Callable, Deque, Dict, Hashable, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Callable, Dict, Hashable
 
 from prompt_toolkit.application.current import get_app
 from prompt_toolkit.cursor_shapes import CursorShape
@@ -34,20 +37,20 @@ __all__ = [
 
 
 def _output_screen_diff(
-    app: "Application[Any]",
+    app: Application[Any],
     output: Output,
     screen: Screen,
     current_pos: Point,
     color_depth: ColorDepth,
-    previous_screen: Optional[Screen],
-    last_style: Optional[str],
+    previous_screen: Screen | None,
+    last_style: str | None,
     is_done: bool,  # XXX: drop is_done
     full_screen: bool,
-    attrs_for_style_string: "_StyleStringToAttrsCache",
-    style_string_has_style: "_StyleStringHasStyleCache",
+    attrs_for_style_string: _StyleStringToAttrsCache,
+    style_string_has_style: _StyleStringHasStyleCache,
     size: Size,
     previous_width: int,
-) -> Tuple[Point, Optional[str]]:
+) -> tuple[Point, str | None]:
     """
     Render the diff between this screen and the previous screen.
 
@@ -139,7 +142,7 @@ def _output_screen_diff(
             write(char.char)
             last_style = char.style
 
-    def get_max_column_index(row: Dict[int, Char]) -> int:
+    def get_max_column_index(row: dict[int, Char]) -> int:
         """
         Return max used column index, ignoring whitespace (without style) at
         the end of the line. This is important for people that copy/paste
@@ -152,13 +155,12 @@ def _output_screen_diff(
         - The `Window` adds a style class to the current line for highlighting
           (cursor-line).
         """
-        numbers = [
+        numbers = (
             index
             for index, cell in row.items()
             if cell.char != " " or style_string_has_style[cell.style]
-        ]
-        numbers.append(0)
-        return max(numbers)
+        )
+        return max(numbers, default=0)
 
     # Render for the first time: reset styling.
     if not previous_screen:
@@ -189,7 +191,6 @@ def _output_screen_diff(
 
     # Loop over the rows.
     row_count = min(max(screen.height, previous_screen.height), height)
-    c = 0  # Column counter.
 
     for y in range(row_count):
         new_row = screen.data_buffer[y]
@@ -200,7 +201,7 @@ def _output_screen_diff(
         previous_max_line_len = min(width - 1, get_max_column_index(previous_row))
 
         # Loop over the columns.
-        c = 0
+        c = 0  # Column counter.
         while c <= new_max_line_len:
             new_char = new_row[c]
             old_char = previous_row[c]
@@ -233,7 +234,7 @@ def _output_screen_diff(
     # output. That way, we're sure that the terminal scrolls up, even when the
     # lower lines of the canvas just contain whitespace.
 
-    # The most obvious reason that we actually want this behaviour is the avoid
+    # The most obvious reason that we actually want this behavior is the avoid
     # the artifact of the input scrolling when the completion menu is shown.
     # (If the scrolling is actually wanted, the layout can still be build in a
     # way to behave that way by setting a dynamic height.)
@@ -274,10 +275,9 @@ class _StyleStringToAttrsCache(Dict[str, Attrs]):
 
     def __init__(
         self,
-        get_attrs_for_style_str: Callable[["str"], Attrs],
+        get_attrs_for_style_str: Callable[[str], Attrs],
         style_transformation: StyleTransformation,
     ) -> None:
-
         self.get_attrs_for_style_str = get_attrs_for_style_str
         self.style_transformation = style_transformation
 
@@ -300,7 +300,7 @@ class _StyleStringHasStyleCache(Dict[str, bool]):
     output if there's no text in the cell.
     """
 
-    def __init__(self, style_string_to_attrs: Dict[str, Attrs]) -> None:
+    def __init__(self, style_string_to_attrs: dict[str, Attrs]) -> None:
         self.style_string_to_attrs = style_string_to_attrs
 
     def __missing__(self, style_str: str) -> bool:
@@ -320,6 +320,7 @@ class _StyleStringHasStyleCache(Dict[str, bool]):
 
 class CPR_Support(Enum):
     "Enum: whether or not CPR is supported."
+
     SUPPORTED = "SUPPORTED"
     NOT_SUPPORTED = "NOT_SUPPORTED"
     UNKNOWN = "UNKNOWN"
@@ -344,9 +345,8 @@ class Renderer:
         output: Output,
         full_screen: bool = False,
         mouse_support: FilterOrBool = False,
-        cpr_not_supported_callback: Optional[Callable[[], None]] = None,
+        cpr_not_supported_callback: Callable[[], None] | None = None,
     ) -> None:
-
         self.style = style
         self.output = output
         self.full_screen = full_screen
@@ -359,23 +359,22 @@ class Renderer:
         self._cursor_key_mode_reset = False
 
         # Future set when we are waiting for a CPR flag.
-        self._waiting_for_cpr_futures: Deque[Future[None]] = deque()
+        self._waiting_for_cpr_futures: deque[Future[None]] = deque()
         self.cpr_support = CPR_Support.UNKNOWN
 
         if not output.responds_to_cpr:
             self.cpr_support = CPR_Support.NOT_SUPPORTED
 
         # Cache for the style.
-        self._attrs_for_style: Optional[_StyleStringToAttrsCache] = None
-        self._style_string_has_style: Optional[_StyleStringHasStyleCache] = None
-        self._last_style_hash: Optional[Hashable] = None
-        self._last_transformation_hash: Optional[Hashable] = None
-        self._last_color_depth: Optional[ColorDepth] = None
+        self._attrs_for_style: _StyleStringToAttrsCache | None = None
+        self._style_string_has_style: _StyleStringHasStyleCache | None = None
+        self._last_style_hash: Hashable | None = None
+        self._last_transformation_hash: Hashable | None = None
+        self._last_color_depth: ColorDepth | None = None
 
         self.reset(_scroll=True)
 
     def reset(self, _scroll: bool = False, leave_alternate_screen: bool = True) -> None:
-
         # Reset position
         self._cursor_pos = Point(x=0, y=0)
 
@@ -383,10 +382,10 @@ class Renderer:
         # we can create a `diff` between two screens and only output the
         # difference. It's also to remember the last height. (To show for
         # instance a toolbar at the bottom position.)
-        self._last_screen: Optional[Screen] = None
-        self._last_size: Optional[Size] = None
-        self._last_style: Optional[str] = None
-        self._last_cursor_shape: Optional[CursorShape] = None
+        self._last_screen: Screen | None = None
+        self._last_size: Size | None = None
+        self._last_style: str | None = None
+        self._last_cursor_shape: CursorShape | None = None
 
         # Default MouseHandlers. (Just empty.)
         self.mouse_handlers = MouseHandlers()
@@ -424,7 +423,7 @@ class Renderer:
         self.output.flush()
 
     @property
-    def last_rendered_screen(self) -> Optional[Screen]:
+    def last_rendered_screen(self) -> Screen | None:
         """
         The `Screen` class that was generated during the last rendering.
         This can be `None`.
@@ -583,7 +582,7 @@ class Renderer:
             task.cancel()
 
     def render(
-        self, app: "Application[Any]", layout: "Layout", is_done: bool = False
+        self, app: Application[Any], layout: Layout, is_done: bool = False
     ) -> None:
         """
         Render the current interface to the output.
@@ -768,8 +767,8 @@ def print_formatted_text(
     output: Output,
     formatted_text: AnyFormattedText,
     style: BaseStyle,
-    style_transformation: Optional[StyleTransformation] = None,
-    color_depth: Optional[ColorDepth] = None,
+    style_transformation: StyleTransformation | None = None,
+    color_depth: ColorDepth | None = None,
 ) -> None:
     """
     Print a list of (style_str, text) tuples in the given style to the output.
@@ -781,7 +780,7 @@ def print_formatted_text(
     # Reset first.
     output.reset_attributes()
     output.enable_autowrap()
-    last_attrs: Optional[Attrs] = None
+    last_attrs: Attrs | None = None
 
     # Print all (style_str, text) tuples.
     attrs_for_style_string = _StyleStringToAttrsCache(
@@ -799,12 +798,16 @@ def print_formatted_text(
                 output.reset_attributes()
         last_attrs = attrs
 
-        # Eliminate carriage returns
-        text = text.replace("\r", "")
-
-        # Assume that the output is raw, and insert a carriage return before
-        # every newline. (Also important when the front-end is a telnet client.)
-        output.write(text.replace("\n", "\r\n"))
+        # Print escape sequences as raw output
+        if "[ZeroWidthEscape]" in style_str:
+            output.write_raw(text)
+        else:
+            # Eliminate carriage returns
+            text = text.replace("\r", "")
+            # Insert a carriage return before every newline (important when the
+            # front-end is a telnet client).
+            text = text.replace("\n", "\r\n")
+            output.write(text)
 
     # Reset again.
     output.reset_attributes()
